import math
import pygplates


# Create a topological model from the topological plate polygon features (can also include deforming networks)
# and rotation file(s).
#topological_model = pygplates.TopologicalModel(
#    [
#        r"C:\Program Files\GPlates\GPlates 2.5.0\GeoData\FeatureCollections\DynamicPolygons\Zahirovic_etal_2022_Feature_Geometries.gpml",
#        r"C:\Program Files\GPlates\GPlates 2.5.0\GeoData\FeatureCollections\DynamicPolygons\Zahirovic_etal_2022_Plate_Boundaries.gpml",
#        r"C:\Program Files\GPlates\GPlates 2.5.0\GeoData\FeatureCollections\DeformingLithosphere\Zahirovic_etal_2022_Deforming_Networks_Active.gpml"
#    ],
#    r"C:\Program Files\GPlates\GPlates 2.5.0\GeoData\FeatureCollections\Rotations\Zahirovic_etal_2022_OptimisedMantleRef_and_NNRMantleRef.rot")

topological_model = pygplates.TopologicalModel(
    [
        r"C:\gplates\src\pygplates\pygplates\test\fixtures\topologies.gpml"
    ],
    r"C:\gplates\src\pygplates\pygplates\test\fixtures\rotations.rot")


# Our geological times will be from 0Ma to 'num_time_steps' Ma (inclusive) in 1 My intervals.
#num_time_steps = 100

converging_mid_ocean_ridge_features = []
diverging_subduction_zone_features = []

# 'time' = 0, 1, 2, ... , 140
#for time in range(0, num_time_steps + 1, 10):
for time in (10,):
    
    # Get a snapshot of our resolved topologies at the current 'time'.
    topological_snapshot = topological_model.topological_snapshot(time)

    # Define a function so we don't have to write the same code twice.
    def calculate_converging_or_diverging_points(
            boundary_section_feature_type,
            find_converging_points):
        """
        Calculate plate boundary statistics along boundary sections with feature type 'boundary_section_feature_type'.
        If 'find_converging_points' is True then find converging points, otherwise find diverging points.
        """
        
        # Calculate statistics along plate boundary sections labelled with the requested feature type.
        plate_boundary_statistics = topological_snapshot.calculate_plate_boundary_statistics(
                math.radians(10),  # 0.5 degree spacing between points
                #first_uniform_point_spacing_radians=0.0,
                include_network_boundaries = True)#boundary_section_feature_type)
        
        # Record points satisfying the converging/diverging criteria (and record their convergence velocities)
        points = []
        convergence_velocities = []
        test_stats = []
        for stat in plate_boundary_statistics:
            # If unable to calculate convergence velocity at the current point then skip it.
            #if math.isnan(stat.convergence_velocity_signed_magnitude):
            #    continue
            
            # See if current point is converging (if 'find_converging_points' is True) or
            # diverging (if 'find_converging_points' is False).
            if ((find_converging_points and stat.convergence_velocity_signed_magnitude > 0) or
                (not find_converging_points and stat.convergence_velocity_signed_magnitude < 0) or
                True):
                points.append(stat.boundary_point)
                convergence_velocities.append(stat.convergence_velocity_signed_magnitude)
                strain_rate = stat.left_plate_strain_rate
                if strain_rate == pygplates.StrainRate.zero:
                    strain_rate = stat.right_plate_strain_rate
                test_stats.append(strain_rate.get_dilatation_rate() * 1e17)

        # If there were no points satisfying the converging/diverging criteria then return early.
        if not points:
            return None
        
        # Create a feature containing the points (and their convergence velocities).
        points_feature = pygplates.Feature()
        # Feature only exists at the current 'time' (for display in GPlates).
        points_feature.set_valid_time(time + 0.5, time - 0.5)
        # Set the geometry as a coverage geometry (ie, a multipoint and scalar values).
        # The convergence velocity scalar values will show up in GPlates as a separate layer.
        points_feature.set_geometry(
            (
                pygplates.MultiPointOnSphere(points),
                {
                    pygplates.ScalarType.create_gpml('ConvergenceVelocity') : convergence_velocities,
                    pygplates.ScalarType.create_gpml('TestStat') : test_stats
                }
            )
        )
        
        return points_feature
    
    # Find converging points along mid-ocean ridges.
    converging_mid_ocean_ridge_points_feature = calculate_converging_or_diverging_points(
        pygplates.FeatureType.gpml_mid_ocean_ridge,
        True)  # find converging points
    if converging_mid_ocean_ridge_points_feature:
        converging_mid_ocean_ridge_features.append(converging_mid_ocean_ridge_points_feature)
    
    # Find diverging points along subduction zones.
    diverging_subduction_zone_points_feature = calculate_converging_or_diverging_points(
        pygplates.FeatureType.gpml_subduction_zone,
        False)  # find diverging points)
    if diverging_subduction_zone_points_feature:
        diverging_subduction_zone_features.append(diverging_subduction_zone_points_feature)

# Write all points at all times along mid-ocean ridges that are converging.
pygplates.FeatureCollection(converging_mid_ocean_ridge_features).write('converging-mid-ocean-ridge-points.gpmlz')

# Write all points at all times along subduction zones that are diverging.
pygplates.FeatureCollection(diverging_subduction_zone_features).write('diverging-subduction-zone-points.gpmlz')
